/*
    NodeHandler.h - Langlo Node parameters
    Version 0.0.2	06/03/2023

    0.0.1 Initial release
    0.0.2 Unspecified updates

    Digital Concepts
    March 2023
    digitalconcepts.net.au
 */
 
#include "NodeHandler.h"

  // Software Revision Level
  
#ifndef SoftwareRevision_def
  #define SoftwareRevision_def
  struct _SoftwareRevision {
		const uint8_t major;			// Major feature release
		const uint8_t minor;			// Minor feature enhancement release
		const uint8_t minimus;		// 'Bug fix' release
  };
#endif

_SoftwareRevision _nodeHandlerRevNumber = {0,0,2};

NodeHandler::NodeHandler() {
}
    
String NodeHandler::softwareRevision() {
	String _softwareRev;
	_softwareRev = String(_nodeHandlerRevNumber.major) + "." + String(_nodeHandlerRevNumber.minor) + "." + String(_nodeHandlerRevNumber.minimus);
	return _softwareRev;
}

void NodeHandler::begin() {
  init();
}

void NodeHandler::init() {
  NH_heartbeatInterval = 60;		// 1 minute
  NH_healthCheckInterval = 300;	// 5 minutes
  NH_deadbeat = 5;					    // Missed heartbeats before declared dead
}

void NodeHandler::setNodeMAC(uint32_t _mac) {
  NH_nodeList[0].mac = _mac;
}

void NodeHandler::setNodeTimer(uint16_t _timer) {
  NH_nodeList[0].timer = _timer;
}

void NodeHandler::setNodeTimerResetValue(uint16_t _timerResetValue) {
  NH_nodeList[0].timerResetValue = _timerResetValue;
}

void NodeHandler::setNodeName(const char * _name) {
  NH_nodeList[0].name = _name;
}

void NodeHandler::setNodeTopic(const char * _topic) {
  NH_nodeList[0].topic = _topic;
}

uint16_t NodeHandler::heartbeatInterval() {
  return NH_heartbeatInterval;
}

void NodeHandler::setHeartbeatInterval(uint16_t _heartbeatInterval) {
  NH_heartbeatInterval = _heartbeatInterval;
}

uint16_t NodeHandler::healthCheckInterval() {
  return NH_healthCheckInterval;
}

void NodeHandler::setHealthCheckInterval(uint16_t _healthCheckInterval) {
  NH_healthCheckInterval = _healthCheckInterval;
}

uint16_t NodeHandler::deadbeat() {
  return NH_deadbeat;
}

void NodeHandler::setDeadbeat(uint16_t _deadbeat) {
  NH_deadbeat = _deadbeat;
}

int NodeHandler::index(uint32_t _sourceMAC) {  
  for (int i = 0; i < NH_nodeCount; i++) {
    if ( NH_nodeList[i].mac == _sourceMAC ) return i;
  }

  // The last element of the array is 'reserved' for the unknown Node
  // If a MAC address does not appear in the list, it is added as the last
  // element. I see now that, the way things are being done now, this will survive
  // the processing of the current packet, which was not the intention—it was meant to
  // just be a temporary store (for some undefined purpose... so maybe I'll just delete
  // this bit in the near future...)
  
  int j = NH_nodeCount - 1;
  NH_nodeList[j].mac = _sourceMAC;
  return j;
}

uint32_t NodeHandler::mac(int _nodeIndex) {
  return NH_nodeList[_nodeIndex].mac;
}

int16_t NodeHandler::timer(int _nodeIndex) {
  return NH_nodeList[_nodeIndex].timer;
}

int16_t NodeHandler::timerResetValue(int _nodeIndex) {
  return NH_nodeList[_nodeIndex].timerResetValue;
}

const String NodeHandler::name(int _nodeIndex) {
  return NH_nodeList[_nodeIndex].name;
}

const String NodeHandler::topic(int _nodeIndex) {
  return NH_nodeList[_nodeIndex].topic;
}

int NodeHandler::nodeCount() {
  return NH_nodeCount;
}

void NodeHandler::incrementTimer(int _nodeIndex) {
  NH_nodeList[_nodeIndex].timer++;
}

void NodeHandler::incrementTimers() {
  for (int i = 0; i < NH_nodeCount; i++) {
    NH_nodeList[i].timer++;
  }
}

void NodeHandler::resetTimer(int _nodeIndex) {
  NH_nodeList[_nodeIndex].timer = NH_nodeList[_nodeIndex].timerResetValue;
}

